package at.ac.uibk.gitsearch.service.criteria;

import java.io.Serializable;
import java.util.Objects;
import tech.jhipster.service.Criteria;
import tech.jhipster.service.filter.Filter;
import tech.jhipster.service.filter.LongFilter;
import tech.jhipster.service.filter.StringFilter;

/**
 * Criteria class for the {@link at.ac.uibk.gitsearch.domain.WatchListEntry} entity. This class is used
 * in {@link at.ac.uibk.gitsearch.web.rest.WatchListEntryResource} to receive all the possible filtering options from
 * the Http GET request parameters.
 * For example the following could be a valid request:
 * {@code /watch-list-entries?id.greaterThan=5&attr1.contains=something&attr2.specified=false}
 * As Spring is unable to properly convert the types, unless specific {@link Filter} class are used, we need to use
 * fix type specific filters.
 */
@SuppressWarnings(
    {
        "PMD.GodClass",
        "PMD.AvoidFieldNameMatchingMethodName",
        "PMD.FieldDeclarationsShouldBeAtStartOfClass",
        "PMD.ShortMethodName",
        "PMD.ConfusingTernary",
    }
)
public class WatchListEntryCriteria implements Serializable, Criteria {

    private static final long serialVersionUID = 1L;

    private LongFilter id;

    private StringFilter exerciseId;

    private StringFilter exerciseName;

    private LongFilter watchlistId;

    private Boolean distinct;

    public WatchListEntryCriteria() {
        super();
    }

    public WatchListEntryCriteria(WatchListEntryCriteria other) {
        this.id = other.id == null ? null : other.id.copy();
        this.exerciseId = other.exerciseId == null ? null : other.exerciseId.copy();
        this.exerciseName = other.exerciseName == null ? null : other.exerciseName.copy();
        this.watchlistId = other.watchlistId == null ? null : other.watchlistId.copy();
        this.distinct = other.distinct;
    }

    @Override
    public WatchListEntryCriteria copy() {
        return new WatchListEntryCriteria(this);
    }

    public LongFilter getId() {
        return id;
    }

    public LongFilter id() {
        if (id == null) {
            id = new LongFilter();
        }
        return id;
    }

    public void setId(LongFilter id) {
        this.id = id;
    }

    public StringFilter getExerciseId() {
        return exerciseId;
    }

    public StringFilter exerciseId() {
        if (exerciseId == null) {
            exerciseId = new StringFilter();
        }
        return exerciseId;
    }

    public void setExerciseId(StringFilter exerciseId) {
        this.exerciseId = exerciseId;
    }

    public StringFilter getExerciseName() {
        return exerciseName;
    }

    public StringFilter exerciseName() {
        if (exerciseName == null) {
            exerciseName = new StringFilter();
        }
        return exerciseName;
    }

    public void setExerciseName(StringFilter exerciseName) {
        this.exerciseName = exerciseName;
    }

    public LongFilter getWatchlistId() {
        return watchlistId;
    }

    public LongFilter watchlistId() {
        if (watchlistId == null) {
            watchlistId = new LongFilter();
        }
        return watchlistId;
    }

    public void setWatchlistId(LongFilter watchlistId) {
        this.watchlistId = watchlistId;
    }

    public Boolean getDistinct() {
        return distinct;
    }

    public void setDistinct(Boolean distinct) {
        this.distinct = distinct;
    }

    @Override
    public boolean equals(Object o) {
        if (this == o) {
            return true;
        }
        if (o == null || getClass() != o.getClass()) {
            return false;
        }
        final WatchListEntryCriteria that = (WatchListEntryCriteria) o;
        return (
            Objects.equals(id, that.id) &&
            Objects.equals(exerciseId, that.exerciseId) &&
            Objects.equals(exerciseName, that.exerciseName) &&
            Objects.equals(watchlistId, that.watchlistId) &&
            Objects.equals(distinct, that.distinct)
        );
    }

    @Override
    public int hashCode() {
        return Objects.hash(id, exerciseId, exerciseName, watchlistId, distinct);
    }

    // prettier-ignore
    @Override
    public String toString() {
        return "WatchListEntryCriteria{" +
            (id != null ? "id=" + id + ", " : "") +
            (exerciseId != null ? "exerciseId=" + exerciseId + ", " : "") +
            (exerciseName != null ? "exerciseName=" + exerciseName + ", " : "") +
            (watchlistId != null ? "watchlistId=" + watchlistId + ", " : "") +
            (distinct != null ? "distinct=" + distinct + ", " : "") +
            "}";
    }
}
