jest.mock('app/login/login.service');

import { ComponentFixture, TestBed, waitForAsync } from '@angular/core/testing';
import { HttpClientTestingModule, HttpTestingController } from '@angular/common/http/testing';
import { RouterTestingModule } from '@angular/router/testing';
import { NgxWebstorageModule } from 'ngx-webstorage';
import { TranslateModule } from '@ngx-translate/core';

import { LoginService } from 'app/login/login.service';

import {
  BadgeAwardedMenuBadgeComponent,
  ImprovementsRequestedMenuBadgeComponent,
  ReviewMenuBadgeComponent,
  ReviewRequestedMenuBadgeComponent,
} from './navbar.component';
import { ReactiveFormsModule } from '@angular/forms';
import { ApplicationConfigService } from 'app/core/config/application-config.service';
import { ReviewManagementService } from 'app/admin/review-management/review-management.service';

const stats = {
  reviewInProgress: 1,
  reviewImproved: 2,
  reviewImprovementRequested: 2,
  reviewCompleted: 3,
  reviewRequested: 13,
  unknownStats: 4,
  reviewsAssignedToReviewer: 5,
  reviewInProgressByReviewer: 6,
  reviewImprovedForReviewer: 7,
  reviewCompletedByReviewer: 8,
  unkownStatsByReviewer: 9,
  reviewRejectedByReviewer: 11,
  reviewImprovementRequestedByReviewer: 12,
  badgesRewarded: 10,
};

describe('ReviewMenuBadgeComponent Component', () => {
  let comp: ReviewMenuBadgeComponent;
  let fixture: ComponentFixture<ReviewMenuBadgeComponent>;
  let httpMock: HttpTestingController;
  let applicationConfigService: ApplicationConfigService;

  beforeEach(
    waitForAsync(() => {
      TestBed.configureTestingModule({
        imports: [
          HttpClientTestingModule,
          RouterTestingModule.withRoutes([]),
          ReactiveFormsModule,
          TranslateModule.forRoot(),
          NgxWebstorageModule.forRoot(),
        ],
        declarations: [ReviewMenuBadgeComponent],
        providers: [LoginService, ReviewManagementService],
      }).compileComponents();
    })
  );

  beforeEach(() => {
    fixture = TestBed.createComponent(ReviewMenuBadgeComponent);
    comp = fixture.componentInstance;
    applicationConfigService = TestBed.inject(ApplicationConfigService);
    httpMock = TestBed.inject(HttpTestingController);
  });

  it('Should call reviewManagementService.getReviewStatistics on init', () => {
    // WHEN
    comp.ngOnInit();
    // WHEN

    const testRequest = httpMock.expectOne({
      method: 'POST',
      url: applicationConfigService.getEndpointFor('api/review/countAllByUser'),
    });
    testRequest.flush(stats);

    expect(comp.getCount()).toEqual(stats.reviewInProgressByReviewer + stats.reviewImprovedForReviewer);
    expect(comp.getLabel()).toEqual(
      `${stats.reviewInProgressByReviewer} item(s) for review.\n${stats.reviewImprovedForReviewer} improved item(s) for review`
    );
  });
});

describe('ReviewRequestedMenuBadge Component', () => {
  let comp: ReviewRequestedMenuBadgeComponent;
  let fixture: ComponentFixture<ReviewRequestedMenuBadgeComponent>;
  let httpMock: HttpTestingController;
  let applicationConfigService: ApplicationConfigService;

  beforeEach(
    waitForAsync(() => {
      TestBed.configureTestingModule({
        imports: [
          HttpClientTestingModule,
          RouterTestingModule.withRoutes([]),
          ReactiveFormsModule,
          TranslateModule.forRoot(),
          NgxWebstorageModule.forRoot(),
        ],
        declarations: [ReviewMenuBadgeComponent],
        providers: [LoginService, ReviewManagementService],
      }).compileComponents();
    })
  );

  beforeEach(() => {
    fixture = TestBed.createComponent(ReviewRequestedMenuBadgeComponent);
    comp = fixture.componentInstance;
    applicationConfigService = TestBed.inject(ApplicationConfigService);
    httpMock = TestBed.inject(HttpTestingController);
  });

  it('Should call reviewManagementService.getReviewStatistics on init', () => {
    // WHEN
    comp.ngOnInit();
    // WHEN

    const testRequest = httpMock.expectOne({
      method: 'POST',
      url: applicationConfigService.getEndpointFor('api/review/countAllByUser'),
    });
    testRequest.flush(stats);

    expect(comp.getCount()).toEqual(stats.reviewRequested);
    expect(comp.getLabel()).toEqual(`${stats.reviewRequested} open review request(s).`);
  });
});

describe('ImprovementsRequestedMenuBadge Component', () => {
  let comp: ImprovementsRequestedMenuBadgeComponent;
  let fixture: ComponentFixture<ImprovementsRequestedMenuBadgeComponent>;
  let httpMock: HttpTestingController;
  let applicationConfigService: ApplicationConfigService;

  beforeEach(
    waitForAsync(() => {
      TestBed.configureTestingModule({
        imports: [
          HttpClientTestingModule,
          RouterTestingModule.withRoutes([]),
          ReactiveFormsModule,
          TranslateModule.forRoot(),
          NgxWebstorageModule.forRoot(),
        ],
        declarations: [ReviewMenuBadgeComponent],
        providers: [LoginService, ReviewManagementService],
      }).compileComponents();
    })
  );

  beforeEach(() => {
    fixture = TestBed.createComponent(ImprovementsRequestedMenuBadgeComponent);
    comp = fixture.componentInstance;
    applicationConfigService = TestBed.inject(ApplicationConfigService);
    httpMock = TestBed.inject(HttpTestingController);
  });

  it('Should call reviewManagementService.getReviewStatistics on init', () => {
    // WHEN
    comp.ngOnInit();
    // WHEN

    const testRequest = httpMock.expectOne({
      method: 'POST',
      url: applicationConfigService.getEndpointFor('api/review/countAllByUser'),
    });
    testRequest.flush(stats);

    expect(comp.getCount()).toEqual(stats.reviewImprovementRequested);
    expect(comp.getLabel()).toEqual(`${stats.reviewImprovementRequested} review(s) resulted in an improvements request`);
  });
});

describe('BadgeAwardedMenuBadge Component', () => {
  let comp: BadgeAwardedMenuBadgeComponent;
  let fixture: ComponentFixture<BadgeAwardedMenuBadgeComponent>;
  let httpMock: HttpTestingController;
  let applicationConfigService: ApplicationConfigService;

  beforeEach(
    waitForAsync(() => {
      TestBed.configureTestingModule({
        imports: [
          HttpClientTestingModule,
          RouterTestingModule.withRoutes([]),
          ReactiveFormsModule,
          TranslateModule.forRoot(),
          NgxWebstorageModule.forRoot(),
        ],
        declarations: [ReviewMenuBadgeComponent],
        providers: [LoginService, ReviewManagementService],
      }).compileComponents();
    })
  );

  beforeEach(() => {
    fixture = TestBed.createComponent(BadgeAwardedMenuBadgeComponent);
    comp = fixture.componentInstance;
    applicationConfigService = TestBed.inject(ApplicationConfigService);
    httpMock = TestBed.inject(HttpTestingController);
  });

  it('Should call reviewManagementService.getReviewStatistics on init', () => {
    // WHEN
    comp.ngOnInit();
    // WHEN

    const testRequest = httpMock.expectOne({
      method: 'POST',
      url: applicationConfigService.getEndpointFor('api/review/countAllByUser'),
    });
    testRequest.flush(stats);

    expect(comp.getCount()).toEqual(10);
    expect(comp.getLabel()).toEqual('10 badge(s) awarded');
  });
});
