import { element, by, ElementFinder } from 'protractor';
/* eslint @typescript-eslint/no-use-before-define: 0 */
export class NavBarPage {
  entityMenu = element(by.id('entity-menu'));
  accountMenu = element(by.id('account-menu'));
  datapolicyCheckbox = element(by.id('datapolicy'));
  adminMenu!: ElementFinder;
  signIn = element(by.id('login'));
  register = element(by.css('[routerLink="account/register"]'));
  signOut = element(by.id('logout'));
  passwordMenu = element(by.css('[routerLink="account/password"]'));
  settingsMenu = element(by.css('[routerLink="account/settings"]'));
  achievementsMenu = element(by.css('[routerLink="account/achievements"]'));
  languageMenu = element(by.id('language'));
  searchMenu = element(by.id('search'));

  constructor(asAdmin?: boolean) {
    if (asAdmin) {
      this.adminMenu = element(by.id('admin-menu'));
    }
  }

  async clickOnEntityMenu(): Promise<void> {
    await this.entityMenu.click();
  }

  async clickOnSearch(): Promise<void> {
    await this.searchMenu.click();
  }

  async clickOnLanguageMenu(): Promise<void> {
    await this.languageMenu.click();
  }

  async clickOnAccountMenu(): Promise<void> {
    await this.accountMenu.click();
  }

  async clickOnAdminMenu(): Promise<void> {
    await this.adminMenu.click();
  }

  async acceptDatapolicy(): Promise<void> {
    await this.datapolicyCheckbox.click();
  }

  async clickOnLanguage(entityName: string): Promise<void> {
    await element(by.id(entityName)).click();
  }

  async clickOnSignIn(): Promise<void> {
    await this.signIn.click();
  }

  async clickOnRegister(): Promise<void> {
    await this.signIn.click();
  }

  async clickOnSignOut(): Promise<void> {
    await this.signOut.click();
  }

  async clickOnPasswordMenu(): Promise<void> {
    await this.passwordMenu.click();
  }

  async clickOnSettingsMenu(): Promise<void> {
    await this.settingsMenu.click();
  }

  async clickOnEntity(entityName: string): Promise<void> {
    await element(by.css('[routerLink="' + entityName + '"]')).click();
  }

  async clickOnAdmin(entityName: string): Promise<void> {
    await element(by.css('[routerLink="admin/' + entityName + '"]')).click();
  }

  getSignInPage(): SignInPage {
    return new SignInPage();
  }

  async goToEntity(entityName: string): Promise<void> {
    await this.clickOnEntityMenu();
    await this.clickOnEntity(entityName);
  }

  async autoSignOut(): Promise<void> {
    await this.clickOnAccountMenu();
    await this.clickOnSignOut();
  }
}

export class SignInPage {
  username = element(by.id('username'));
  password = element(by.id('password'));
  loginButton = element(by.id('submit'));
  privacyCheckbox = element(by.id('datapolicy2'));

  async setUserName(username: string): Promise<void> {
    await this.clearUserName();
    await this.username.sendKeys(username);
  }

  async getUserName(): Promise<string> {
    return this.username.getAttribute('value');
  }

  async clearUserName(): Promise<void> {
    await this.username.clear();
  }

  async setPassword(password: string): Promise<void> {
    await this.clearPassword();
    await this.password.sendKeys(password);
  }

  async getPassword(): Promise<string> {
    return this.password.getAttribute('value');
  }

  async clearPassword(): Promise<void> {
    await this.password.clear();
  }

  async autoSignInUsing(username: string, password: string): Promise<void> {
    await this.setUserName(username);
    await this.setPassword(password);
    await this.login();
  }

  async login(): Promise<void> {
    await this.privacyCheckbox.click();
    await this.loginButton.click();
  }
}
